#![no_std]      // 不链接 Rust 标准库
#![no_main]     // 禁用 main 入口点，因为没有运行时

use core::panic::PanicInfo;

// 定义一个 byte string 类型的静态变量
static HELLO: &[u8] = b"Hello World!";

#[no_mangle]    // 不重整函数名，否则编译器可能会生成名为 _ZN3blog_os4_start7hb173fedf945531caE 的函数
// extern "C" 告诉编译器这个函数应当使用 C 语言的调用约定
// 函数名为 _start 是因为大多数系统默认用这个名字作为入口点名称
// -> ! 代表这是一个发散函数，不允许有任何返回值，因为它不会被任何函数调用，而是将直接被 bootloader 调用
pub extern "C" fn _start() -> ! {
    // VGA Buffer 从 0xb8000 开始
    let vga_buffer = 0xb8000 as *mut u8;

    // 迭代 HELLO, 通过裸指针 vga_buffer 来定位待写入的缓冲区地址
    for (i, &byte) in HELLO.iter().enumerate() {
        unsafe {
            *vga_buffer.offset(i as isize * 2) = byte;      // ASCII 码字节
            *vga_buffer.offset(i as isize * 2 + 1) = 0xb;   // 颜色字节, 0xb 代表青色
        }
    }

    loop {}
}

#[panic_handler]
// panic! 时将会进行栈展开，执行各种 drop 函数来回收垃圾
// 禁用标准库之后，这些东西就都没有了，我们需要把它重写一遍
fn panic(_info: &PanicInfo) -> ! {
    loop {}
}
